unit DynamicObjectsU;

interface

uses
  IniFiles,
  SysUtils, Types, Classes, QGraphics, QControls, QForms, QDialogs,
  QStdCtrls, QExtCtrls;

type
  TForm1 = class(TForm)
    btnCreate: TButton;
    Panel1: TPanel;
    Label1: TLabel;
    btnDestroy: TButton;
    btnColour: TButton;
    dlgColour: TColorDialog;
    procedure btnCreateClick(Sender: TObject);
    procedure btnDestroyClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure btnColourClick(Sender: TObject);
  private
    { Private declarations }
    MyEdit: TEdit;
    IniFile: TIniFile;
    procedure MyEditKeyPress(Sender: TObject; var Key: Char);
    procedure MyEditExit(Sender: TObject);
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.xfm}

procedure TForm1.btnCreateClick(Sender: TObject);
begin
  if MyEdit = nil then
  begin
    MyEdit := TEdit.Create(Self);
    with MyEdit do
    begin
      Text := TimeToStr(Time);
      Color := clLime;
      Font.Color := clRed;
      Top := 24;
      Left := 8;
      MaxLength := 25;
      ShowHint := True;
      Hint := 'I am a dynamically created edit control';
      Parent := Panel1;
      OnKeyPress := MyEditKeyPress;
      OnExit := MyEditExit
    end
  end
end;

procedure TForm1.MyEditKeyPress(Sender: TObject; var Key: Char);
begin
  if Key in ['x', 'X'] then
    Key := #0
end;

procedure TForm1.MyEditExit(Sender: TObject);
var
  Edit: TEdit;
begin
  Edit := Sender as TEdit;
  if Pos('X', UpperCase(Edit.Text)) > 0 then
    Edit.Text := StringReplace(Edit.Text, 'x', '', [rfReplaceAll, rfIgnoreCase])
end;

procedure TForm1.btnDestroyClick(Sender: TObject);
begin
  MyEdit.Free;
  MyEdit := nil
end;

const
  FormSettings = 'Form Settings';
  FormColour = 'Form Colour';

procedure TForm1.FormCreate(Sender: TObject);
begin
  IniFile := TIniFile.Create(GetEnvironmentVariable('HOME') + '/.DynamicObjects');
  Color := StringToColor(IniFile.ReadString(FormSettings, FormColour, ColorToString(Color)));
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  IniFile.UpdateFile; //flush changes to disk
  IniFile.Free; //destroy object
  IniFile := nil //reset reference to nil
end;

procedure TForm1.btnColourClick(Sender: TObject);
begin
  dlgColour.Color := Color; //initialise dialog to show form's colour
  if dlgColour.Execute then
  begin
    Color := dlgColour.Color;
    IniFile.WriteString(FormSettings, FormColour, ColorToString(Color));
  end
end;

end.
